# Generated by Django 2.2 on 2022-02-12 16:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0003_auto_20220212_1614'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='conditiongroup',
            name='grid',
        ),
        migrations.AddField(
            model_name='grid',
            name='condition_group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.ConditionGroup'),
        ),
    ]



# Generated by Django 2.2 on 2023-02-10 10:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0007_auto_20220610_0642'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('comment', models.CharField(blank=True, default='', max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('comment', models.CharField(blank=True, default='', max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='TemplateItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('type', models.IntegerField(default=0)),
                ('order', models.IntegerField(default=0)),
                ('comment', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('optional', models.BooleanField(default=False)),
                ('template', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.Template')),
            ],
        ),
        migrations.CreateModel(
            name='DocumentItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('value', models.CharField(blank=True, default='', max_length=255, null=True)),
                ('document', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.Document')),
            ],
        ),
        migrations.AddField(
            model_name='document',
            name='template',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='django.Template'),
        ),
    ]



from foledol.django.logger import log


class Form:
    def __init__(self, context, fields, namespace=None):
        self.context = context
        self.fields = fields
        for field in fields:
            field.attach(self)
        self.namespace = namespace
        self.old_values = None
        self.new_values = None

    def field(self, key):
        try:
            return next(field for field in self.fields if field.key == key)
        except StopIteration:
            raise Exception("Invalid field '" + key + "'")

    def read(self, values, sibling=None, defaults=None):
        self.old_values = {}
        for field in self.fields:
            self.context[field.full_name] = field.value
            default_value = None
            if defaults and field.key in defaults:
                default_value = defaults[field.key]
            elif sibling and field.key in sibling.__dict__:
                default_value = sibling.__dict__[field.key]
            self.old_values[field.key] = default_value
            # TODO: move this before the self.context
            field.read(values, default_value)
        self.context['form'] = self

    def validate(self):
        self.context['error'] = False
        for field in self.fields:
            field.validate()
            self.context[field.full_name] = field.value
            self.context['errors_on_' + field.key] = None
            if len(field.errors) > 0:
                self.context['errors_on_' + field.key] = field.errors
            self.context['error'] |= field.has_error()

    def save(self, sibling):
        self.new_values = {}
        for field in self.fields:
            if field.key in sibling.__dict__:
                sibling.__dict__[field.key] = field.value
            self.new_values[field.key] = field.value

    def save_and_log(self, sibling, model, action, request):
        self.save(sibling)
        sibling.save()
        log(sibling.id, model, action, request.user, self.old_values, self.new_values)



# Generated by Django 2.2 on 2022-06-10 06:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0006_auto_20220610_0534'),
    ]

    operations = [
        migrations.RenameField(
            model_name='column',
            old_name='sort',
            new_name='order_by',
        ),
    ]



from django import template
from django.urls import reverse

from foledol.django.tools.form import Form

register = template.Library()


translatable = {
    'table.search': "Rechercher"
}


@register.filter
def translate(label):
    return translatable[label]


@register.filter
def get(obj, column):
    try:
        if column.value:
            return value(obj, column.value)
        return value(obj, column.key)
    except AttributeError:
        return None


@register.filter
def value(obj, name):
    split = name.split('.', 1)
    if len(split) > 1:
        return value(getattr(obj, split[0]), split[1]) if (obj and hasattr(obj, split[0])) else None
    return getattr(obj, name) if (obj and hasattr(obj, name)) else None


@register.filter
def value_as_str(obj, name):
    split = name.split('.', 1)
    if len(split) > 1:
        return value(getattr(obj, split[0]), split[1]) if (obj and hasattr(obj, split[0])) else None
    as_str = value(obj, name + '_as_str')
    return as_str() if as_str else value(obj, name)


@register.filter
def method(obj, name):
    split = name.split('.', 1)
    if len(split) > 1:
        return method(getattr(obj, split[0]), split[1]) if (obj and hasattr(obj, split[0])) else None
    return getattr(obj, name)() if (obj and hasattr(obj, name)) else None


@register.filter
def formatter(table, row):
    return table.formatter(row)


@register.filter
def field(form, key):
    return form.field(key)


@register.filter
def query(dictionary):
    query_str = ''
    for key in dictionary.keys():
        if len(query_str) > 0:
            query_str += "&"
        query_str += key + "=" + str(dictionary[key])
    return query_str


@register.filter
def report(reports, key):
    return reports[key]


@register.filter
def editor(row, fields):
    return Form(context={}, fields=fields)


@register.filter
def class_name(obj):
    return type(obj).__name__


links = {}


@register.filter
def link(obj, name):
    return links[name](obj)


def register_link(name):
    def register_link_inner(function):
        links[name] = function
    return register_link_inner


@register.filter
def resolve(name, pk):
    return reverse(name, kwargs={'pk': pk}) if pk > 0 else None
